; Fast Keyboard v1.2
; Dated 5-1-00 by Jeff Flanigan
; Prog to speed up the autorepeat of the keyboard

_nostub
_ti89
_ti92plus
		xdef		_nostub
		xdef		_ti89
		xdef		_ti92plus
		xdef		_main

OSInitKeyInitDelay		equ	$248
OSInitBetweenKeyDelay		equ	$249
ST_helpMsg			equ	$E6
HeapAllocHigh			equ	$92
HLock				equ	$99
HeapPtrToHandle			equ	$23A
HeapFree			equ	$97

ROM_CALL	macro
		move.l		($C8),a5
		add.l		#\1*4,a5
		move.l		(a5),a5
		jsr		(a5)
		endm

; ------------------------- Start of Program -----------------------

_main:

; Test to see if installed
testInstalled:
		move.l		$90,a0
		lea		newTrap4(PC),a1
		move.l		#(oldTrap4-newTrap4)/2-1,d0
\loop:
		cmpm.w		(a0)+,(a1)+
		bne		install
		dbf		d0,\loop	; same as dbra
		bra		uninstall

; ----------------------------- Install ----------------------------

install:
; Show installed message
		pea		installed
		ROM_CALL	ST_helpMsg
		addq.l		#4,a7

; Set autorepeat delay
		move.w		#100,-(a7)
		ROM_CALL	OSInitKeyInitDelay
		move.w		#20,(a7)
		ROM_CALL	OSInitBetweenKeyDelay
		addq.l		#2,a7

; Allocate memory for new trap 4
		move.l		#newTrap4End-newTrap4,-(a7)
		ROM_CALL	HeapAllocHigh
		addq.l		#4,a7
		tst.w		d0
		bne		\AllocOK
; If error show error message and exit
		pea		errormem
		ROM_CALL	ST_helpMsg
		addq.l		#4,a7
		rts

; Lock and dereference the handle
\AllocOK:
		move.w		d0,-(a7)
		ROM_CALL	HLock	; lock and deref
		addq.l		#2,a7
		move.l		a0,d0
		tst.l		d0
		bne		\LockOK
; If error show error message and exit
		pea		errorinstall
		ROM_CALL	ST_helpMsg
		addq.l		#4,a7
		rts

; Copy new trap 4 routine to allocated memory
\LockOK:
		move.l		a0,-(a7)	; save ptr
		lea		newTrap4(PC),a1
		move.l		#(newTrap4End-newTrap4)/2-1,d0
\loop:
		move.w		(a1)+,(a0)+
		dbf		d0,\loop	; same as dbra
		move.l		(a7),a0		; restore ptr but
						; keep ptr saved

; Save old trap 4 address
		add.l		#oldTrap4-newTrap4,a0
		move.l		$90,(a0)

; Set up trap 4 vector
		move.l		(a7)+,d0	; restore ptr
		bclr		#2,$600001
		move.l		d0,$90
		bset		#2,$600001

		rts

; -------------------------- Uninstall -----------------------------

uninstall:
; Set default autorepeat
		move.w		#336,-(a7)
		ROM_CALL	OSInitKeyInitDelay
		move.w		#48,(a7)
		ROM_CALL	OSInitBetweenKeyDelay
		addq.l		#2,a7

; Restore old trap 4 vector
		move.l		$90,d0
		move.l		d0,-(a7)	; save ptr
		add.l		#oldTrap4-newTrap4,d0
		move.l		d0,a0
		bclr		#2,$600001
		move.l 		(a0),$90
		bset		#2,$600001

; Get handle of trap 4
		; ptr already pushed
		ROM_CALL	HeapPtrToHandle
		addq.l		#4,a7
		tst.w		d0
		bne		\PtrToHandleOK
; If error show error message and exit
		pea		erroruninstall
		ROM_CALL	ST_helpMsg
		addq.l		#4,a7
		rts

; Free handle of trap 4
\PtrToHandleOK:
		move.w		d0,-(a7)
		ROM_CALL	HeapFree
		addq.l		#2,a7

; Show uninstalled message
		pea		uninstalled
		ROM_CALL	ST_helpMsg
		addq.l		#4,a7

		rts

; ------------------------- New Trap #4 ----------------------------

newTrap4:
		movem.l		d0-d7/a0-a6,-(a7)

; Restore default trap 4
		move.l		oldTrap4(PC),a0
		bclr		#2,$600001
		move.l 		a0,$90
		bset		#2,$600001

; Turn off calc
		movem.l		(a7)+,d0-d7/a0-a6
		trap		#4
		movem.l		d0-d7/a0-a6,-(a7)

; Restore the new trap 4
		lea		newTrap4(PC),a0
		bclr		#2,$600001
		move.l 		a0,$90
		bset		#2,$600001

; Set autorepeat delay
; (calc inits this slow; must be reinited to fast)
		move.w		#100,-(a7)
		ROM_CALL	OSInitKeyInitDelay
		move.w		#20,(a7)
		ROM_CALL	OSInitBetweenKeyDelay
		addq.l		#2,a7

		movem.l		(a7)+,d0-d7/a0-a6
		rte

oldTrap4:
		dc.l		0

newTrap4End

; -------------------------- Data ----------------------------------

installed:
		dc.b  "Fastkbd v1.2 by Jeff Flanigan installed!",0
uninstalled:
		dc.b		"Fast Keyboard uninstalled",0
errormem:
		dc.b	"Error: Not enough mem to install",0
errorinstall:
		dc.b	"Error occured during installation",0
erroruninstall:
		dc.b	"Error occured during uninstallation",0

		end